@@ -5,6 +5,8 @@ from typing import List, Optional, Tuple
 import numpy as np  # type: ignore
 import pandas as pd  # type: ignore
 from pandas.core.generic import NDFrame # type:ignore
+import dask
+import dask.dataframe as dd
 
 #from tscv import GapWalkForward # type: ignore
 from sklearn.model_selection import TimeSeriesSplit
@@ -14,7 +16,7 @@ from sklearn.model_selection import (ShuffleSplit, StratifiedShuffleSplit, # typ
                                     TimeSeriesSplit, cross_val_score) # type: ignore
 from sklearn.discriminant_analysis import LinearDiscriminantAnalysis # type: ignore
 from sklearn.ensemble import (BaggingRegressor, ExtraTreesRegressor,  # type: ignore
-                             RandomForestClassifier, ExtraTreesClassifier,  # type: ignore
+                             BaggingClassifier, ExtraTreesClassifier,  # type: ignore
                              AdaBoostRegressor, AdaBoostClassifier, # type: ignore
                              RandomForestClassifier, RandomForestRegressor) # type: ignore
 
@@ -31,6 +33,7 @@ from .build_base import BuildBase
 # helper functions
 from ..utils import print_static_rmse, print_dynamic_rmse, convert_timeseries_dataframe_to_supervised, print_ts_model_stats
 import pdb
+import time
 
 class BuildML(BuildBase):
     def __init__(self, scoring: str = '', forecast_period: int = 2, ts_column: str = '', verbose: int = 0):
@@ -55,7 +58,7 @@ class BuildML(BuildBase):
         # This saves the last `self.lags` of the original train dataframe
         # This is needed during predictions to transform the X_test
         # to a supervised learning problem.
-        self.df_train_prepend: pd.DataFrame = pd.DataFrame()
+        self.df_train_prepend = pd.DataFrame()
         self.train_df = pd.DataFrame()
 
 
@@ -64,7 +67,7 @@ class BuildML(BuildBase):
         Build a Time Series Model using Machine Learning models.
         Quickly builds and runs multiple models for a clean data set (only numerics).
         """
-
+        ts_df = copy.deepcopy(ts_df)
         self.original_target_col = target_col
         self.lags = lags
         self.original_preds = [x for x in list(ts_df) if x not in [self.original_target_col]]
@@ -72,13 +75,21 @@ class BuildML(BuildBase):
 
         if len(self.original_preds) > 0:
             self.univariate = False
-            features_dict = classify_features(ts_df, self.original_target_col)
-            cols_to_remove = features_dict['cols_delete'] + features_dict['IDcols'] + features_dict['discrete_string_vars']
-            preds = [x for x in list(ts_df) if x not in [self.original_target_col]+cols_to_remove]
-            catvars = ts_df[preds].select_dtypes(include = 'object').columns.tolist() + ts_df[preds].select_dtypes(include = 'category').columns.tolist()
+            if type(ts_df) == dask.dataframe.core.DataFrame:
+                continuous_vars = ts_df.select_dtypes('number').columns.tolist()
+                numvars = [x for x in continuous_vars if x not in [target_col]]
+                preds = [x for x in list(ts_df) if x not in [self.original_target_col]]
+                catvars = ts_df.select_dtypes('object').columns.tolist() + ts_df.select_dtypes('category').columns.tolist()
+                if len(catvars) > 0:
+                    print('    Warning: Dropping Categorical variables %s. You can Label Encode them and try ML modeling again...' %catvars)
+            else:
+                features_dict = classify_features(ts_df, self.original_target_col)
+                cols_to_remove = features_dict['cols_delete'] + features_dict['IDcols'] + features_dict['discrete_string_vars']
+                preds = [x for x in list(ts_df) if x not in [self.original_target_col]+cols_to_remove]
+                catvars = ts_df[preds].select_dtypes(include = 'object').columns.tolist() + ts_df[preds].select_dtypes(include = 'category').columns.tolist()
+                numvars = ts_df[preds].select_dtypes(include = 'number').columns.tolist()
             if len(catvars) > 0:
-                print('    Warning: Dropping Categorical variables from analysis. You can Label Encode them and try ML modeling again...')
-            numvars = ts_df[preds].select_dtypes(include = 'number').columns.tolist()
+                print('    Warning: Dropping Categorical variables %s. You can Label Encode them and try ML modeling again...' %catvars)
             self.original_preds = numvars
             preds = copy.deepcopy(numvars)
             if len(numvars) > 30:
@@ -87,19 +98,25 @@ class BuildML(BuildBase):
             self.univariate = True
             preds = self.original_preds[:]
 
-
         ts_df = ts_df[preds+[self.original_target_col]]
 
         # Order data
         ts_df = self.order_df(ts_df)
-        num_obs = ts_df.shape[0]
-        self.train_df = ts_df
+
+        if type(ts_df) == dask.dataframe.core.DataFrame:
+            num_obs = ts_df.shape[0].compute()
+        else:
+            num_obs = ts_df.shape[0]
+
+        #self.train_df = ts_df  # you don't want to store the entire train_df in model
         # Convert to supervised learning problem
         dfxs, self.transformed_target, self.transformed_preds = self.df_to_supervised(
                 ts_df=ts_df, drop_zero_var = True)
 
-        print("Fitting ML model")
-        print('\n    List of variables used in training Model = %s' %self.transformed_preds)
+
+        print("\nFitting ML model")
+        print('    %d variables used in training ML model = %s' %(
+                                len(self.transformed_preds),self.transformed_preds))
         if len(self.transformed_preds) > 1:
             self.univariate = False
         else:
@@ -117,7 +134,11 @@ class BuildML(BuildBase):
         rmse_folds = []
         norm_rmse_folds = []
         forecast_df_folds = []  # TODO: See if this can be retreived somehow
-
+        ### Creating a new way to skip cross validation when trying to run auto-ts multiple times. ###
+        if cv == 0:
+            cv_in = 0
+        else:
+            cv_in = copy.deepcopy(cv)
         NFOLDS = self.get_num_folds_from_cv(cv)
         ###########################################################################
         if self.forecast_period <= 5:
@@ -126,7 +147,7 @@ class BuildML(BuildBase):
         ### In case the number of forecast_period is too high, just reduce it so it can fit into num_obs
         if NFOLDS*self.forecast_period > num_obs:
             self.forecast_period = int(num_obs/10)
-            print('    Cutting forecast period by 90 percent to %d to enable cross_validation' %self.forecast_period)
+            print('    Forecast period too high. Cutting by 90%% to %d to enable cross_validation' %self.forecast_period)
         ###########################################################################
 
         seed = 99
@@ -147,94 +168,141 @@ class BuildML(BuildBase):
             self.scoring = 'neg_root_mean_squared_error'
 
         print('\nRunning Cross Validation using XGBoost model..')
-        ################################################################################
-        num_obs = dft.shape[0]
-        if self.forecast_period <= 5:
-            #### Set a minimum of 5 for the number of rows in test!
-            self.forecast_period = 5
-        ### In case the number of forecast_period is too high, just reduce it so it can fit into num_obs
-        if NFOLDS*self.forecast_period > num_obs:
-            self.forecast_period = int(num_obs/(NFOLDS+1))
-            print('Lowering forecast period to %d to enable cross_validation' %self.forecast_period)
         ###########################################################################################
         #cv = GapWalkForward(n_splits=NFOLDS, gap_size=0, test_size=self.forecast_period)
         max_trainsize = len(dft) - self.forecast_period
-        cv = TimeSeriesSplit(n_splits=NFOLDS, max_train_size = max_trainsize)
-        y_preds = pd.DataFrame()
+        try:
+            cv = TimeSeriesSplit(n_splits=NFOLDS, test_size=self.forecast_period) ### this works only sklearn v 0.0.24]
+        except:
+            cv = TimeSeriesSplit(n_splits=NFOLDS, max_train_size = max_trainsize)
+
         print('Max. iterations using expanding window cross validation = %d' %NFOLDS)
         start_time = time.time()
         rmse_folds = []
         norm_rmse_folds = []
         y_trues = copy.deepcopy(y_train)
+        concatenated = pd.DataFrame()
+        extra_concatenated = pd.DataFrame()
 
-        for fold_number, (train_index, test_index) in enumerate(cv.split(dft)):
-            train_fold = dft.iloc[train_index]
-            test_fold = dft.iloc[test_index]
-            horizon = len(test_fold)
+        if type(dft) == dask.dataframe.core.DataFrame:
+            dft = dft.head(len(dft)) ### this converts dask into a pandas dataframe
+        #### If cross validation is zero, then it means cross validation needs to be skipped
+        if  cv_in == 0:
+            print('Skipping cross validation steps since cross_validation = %s' %cv_in)
+            try:
+                model_name = 'XGBoost'
+                model = XGBRegressor(n_estimators=400, verbosity=None, random_state=0)
+            except:
+                model_name = 'Random Forest'
+                model = RandomForestRegressor(n_estimators=200, random_state=99)
+        else:
+            for fold_number, (train_index, test_index) in enumerate(cv.split(dft)):
+                dftx = dft.head(len(train_index)+len(test_index))
+                train_fold = dftx.head(len(train_index)) ## now train will be the first segment of dftx
+                test_fold = dftx.tail(len(test_index)) ### now test will be right after train in dftx
 
-            print(f"\nFold Number: {fold_number+1} --> Train Shape: {train_fold.shape[0]} Test Shape: {test_fold.shape[0]}")
+                horizon = len(test_fold)
 
-            #########################################
-            #### Define the model with fold data ####
-            #########################################
+                print(f"\nFold Number: {fold_number+1} --> Train Shape: {train_fold.shape[0]} Test Shape: {test_fold.shape[0]}")
 
-            #model = RandomForestRegressor(n_estimators=NUMS, random_state=99)
-            model = XGBRegressor(n_estimators=400, verbosity=None, random_state=0)
-            nums = int(0.9*train_fold.shape[0])
-            if nums <= 1:
-                nums = 2
-            ############################################
-            #### Fit the model with train_fold data ####
-            ############################################
+                #########################################
+                #### Define the model with fold data ####
+                #########################################
+                ## If XGBoost is present in machine, use it. Otherwise use RandomForestRegressor
+                try:
+                    model_name = 'XGBoost'
+                    model = XGBRegressor(n_estimators=400, verbosity=None, random_state=0)
+                except:
+                    model_name = 'Random Forest'
+                    model = RandomForestRegressor(n_estimators=200, random_state=99)
 
-            X_train_fold, y_train_fold = train_fold[:nums][self.transformed_preds], train_fold[:nums][self.transformed_target]
-            X_test_fold, y_test_fold = train_fold[nums:][self.transformed_preds], train_fold[nums:][self.transformed_target]
+                if type(dft) == dask.dataframe.core.DataFrame:
+                    nums = int(0.9*len(train_index))
+                else:
+                    nums = int(0.9*train_fold.shape[0])
 
-            #model.fit(train_fold[self.transformed_preds], train_fold[self.transformed_target])
-            model.fit(X_train_fold, y_train_fold,
-                    eval_set=[(X_train_fold, y_train_fold), (X_test_fold, y_test_fold)],
-                    early_stopping_rounds=50,verbose=False,
-                    )
-            #################################################
-            #### Predict using model with test_fold data ####
-            #################################################
+                if nums <= 1:
+                    nums = 2
+                ############################################
+                #### Fit the model with train_fold data ####
+                ############################################
 
-            forecast_df = model.predict(test_fold[self.transformed_preds])
+                X_train_fold, y_train_fold = train_fold[:nums][self.transformed_preds], train_fold[:nums][self.transformed_target]
+                X_test_fold, y_test_fold = train_fold[nums:][self.transformed_preds], train_fold[nums:][self.transformed_target]
+                if model_name == 'XGBoost':
+                    model.fit(X_train_fold, y_train_fold,
+                        eval_set=[(X_train_fold, y_train_fold), (X_test_fold, y_test_fold)],
+                        early_stopping_rounds=50,verbose=False,
+                        )
+                else:
+                    ## This is for Random Forest regression
+                    model.fit(X_train_fold, y_train_fold)
+                #################################################
+                #### Predict using model with test_fold data ####
+                #################################################
 
-            ### Now compare the actuals with predictions ######
-            y_pred = forecast_df[-horizon:]
-            if fold_number == 0:
-                y_preds = pd.DataFrame(y_pred, columns=['Predicted'])
-            else:
-                y_preds = y_preds.append(pd.DataFrame(y_pred, columns=['Predicted'])).reset_index(drop=True)
-            rmse_fold, rmse_norm = print_dynamic_rmse(test_fold[self.transformed_target],y_pred,test_fold[self.transformed_target])
-            print('Cross Validation window: %d completed' %(fold_number+1,))
-            rmse_folds.append(rmse_fold)
-            norm_rmse_folds.append(rmse_norm)
+                forecast_df = model.predict(test_fold[self.transformed_preds])
+                forecast_df_folds.append(forecast_df)
 
-        ######################################################
-        ### This is where you consolidate the CV results #####
-        ######################################################
-        _ = plot_importance(model, height=0.9,importance_type='gain', title='XGBoost Feature Importance by Gain')
-        rmse_mean = np.mean(rmse_folds)
-        cv_size = y_preds.values.ravel().shape[0]
-        print_ts_model_stats(y_trues.values[-cv_size:], y_preds.values.ravel(),"Random Forest")
-        print('Average CV RMSE over %d windows (macro) = %0.5f' %(fold_number+1,rmse_mean))
-        cv_micro = np.sqrt(mean_squared_error(y_trues.values[-cv_size:], y_preds.values.ravel()))
-        print('Average CV RMSE of all predictions (micro) = %0.5f' %cv_micro)
+                ### Now compare the actuals with predictions ######
+                y_pred = forecast_df[-horizon:]
+                ### y_pred is already an array - so keep that in mind!
 
+                concatenated = pd.DataFrame(np.c_[test_fold[self.transformed_target].values,
+                            y_pred], columns=['original', 'predicted'],index=test_fold.index)
+
+                if fold_number == 0:
+                    extra_concatenated = copy.deepcopy(concatenated)
+                else:
+                    extra_concatenated = extra_concatenated.append(concatenated)
+
+                rmse_fold, rmse_norm = print_dynamic_rmse(concatenated['original'].values, concatenated['predicted'].values,
+                                            concatenated['original'].values)
+
+                print('Cross Validation window: %d completed' %(fold_number+1,))
+                rmse_folds.append(rmse_fold)
+                norm_rmse_folds.append(rmse_norm)
+            ######################################################
+            ### This is where you consolidate the CV results #####
+            ######################################################
+            try:
+                _ = plot_importance(model, height=0.9,importance_type='gain', title='%s Feature Importance by Gain' %model_name)
+
+                if type(y_trues) == dask.dataframe.core.DataFrame or type(y_trues) == dask.dataframe.core.Series:
+                    y_trues = y_trues.head(len(y_trues))
+
+                cv_micro, cv_micro_pct = print_ts_model_stats(extra_concatenated['original'], extra_concatenated['predicted'], model_name)
+
+                print('Average CV RMSE of all predictions (micro) = %0.5f' %cv_micro)
+                #print('Normalized RMSE (as Std Dev of Actuals - micro) = %0.0f%%' %cv_micro_pct)
+            except:
+                print('Could not plot ML results due to error. Continuing...')
         ###############################################
         #### Refit the model on the entire dataset ####
         ###############################################
         self.model = model
-
+        print('\nFitting model on entire train set. Please be patient...')
+        start_time = time.time()
         # Refit Model on entire train dataset (earlier, we only trained the model on the individual splits)
         # Convert to supervised learning problem
-        if self.univariate:
-            self.model.fit(X_train, y_train)
-        else:
-            self.refit(ts_df)
 
+        if self.univariate:
+            if type(X_train) == dask.dataframe.core.DataFrame or type(X_train) == dask.dataframe.core.Series:
+                    X_train = X_train.head(len(X_train)) ## this converts it to a pandas dataframe
+                    self.model.fit(X_train, y_trues)
+            else:
+                self.model.fit(X_train, y_train)
+        else:
+            if type(X_train) == dask.dataframe.core.DataFrame:
+                    X_train = X_train.head(len(X_train)) ## this converts it to a pandas dataframe
+                    self.model.fit(X_train, y_trues)
+            else:
+                self.model.fit(X_train, y_train)
+
+        # Save last `self.lags` which will be used for predictions later
+        self.df_train_prepend = ts_df[-self.lags:]
+
+        print('    Time taken to train model (in seconds) = %0.0f' %(time.time()-start_time))
         # # This is the new method without the leakage
         # # Drop the y value
         # test_orig_df_pred_only = test_orig_df.drop(self.original_target_col, axis=1, inplace=False)
@@ -275,19 +343,28 @@ class BuildML(BuildBase):
         into a supervised learning problem.
         rtype: pd.DataFrame, str, List[str]
         """
+        if self.lags <= 1:
+            n_in = 1
+        elif self.lags >= 4:
+            n_in = 4
+        else:
+            n_in = 1
+
+        self.lags = copy.deepcopy(n_in)
 
         dfxs, transformed_target_name, _ = convert_timeseries_dataframe_to_supervised(
             ts_df[self.original_preds+[self.original_target_col]],
             self.original_preds+[self.original_target_col],
             self.original_target_col,
-            n_in=self.lags, n_out=0, dropT=False
-        )
+            n_in=n_in, n_out=0, dropT=False
+                            )
 
         # Append the time series features (derived from the time series index)
-        # None ts_column will use the index
-        dfxs = create_time_series_features(dfxs, transformed_target_name, ts_column=None, drop_zero_var=drop_zero_var)
+        dfxs = create_ts_features_dask(df=dfxs, tscol=self.ts_column, drop_zero_var=False, return_original=True)
+        #dfxs = create_time_series_features(dfxs, transformed_target_name, ts_column=None, drop_zero_var=drop_zero_var)
 
         # Overwrite with new ones
+
         # transformed_pred_names = [x for x in list(dfxs) if x not in [self.transformed_target]]
         transformed_pred_names = [x for x in list(dfxs) if x not in [transformed_target_name]]
 
@@ -329,7 +406,7 @@ class BuildML(BuildBase):
         to get the forecast period.
         """
         print('For large datasets: ML predictions will take time since it has to predict each row and use that for future predictions...')
-
+        testdata = copy.deepcopy(testdata)
         self.check_model_built()
 
         if testdata is None:
@@ -375,6 +452,9 @@ class BuildML(BuildBase):
             testdata_with_dummy = self.order_df(testdata_with_dummy)
             # print (f"Columns after reordering: {testdata_with_dummy.columns}")
 
+            df_prepend = self.df_train_prepend.copy(deep=True)
+            df_prepend = self.order_df(df_prepend)
+
             # STEP 2:
             # Make prediction for each row. Then use the prediction for the next row.
 
@@ -390,21 +470,38 @@ class BuildML(BuildBase):
             # print("Index After")
             # print(index)
 
-            df_prepend = self.df_train_prepend.copy(deep=True)
+
+            #### Leave this out since we don't want to store the entire train_df in the model
+            #if type(self.train_df) == dask.dataframe.core.DataFrame:
+            #    traindf = self.train_df.head(self.forecast_period) ## convert from DASK to pandas
+            #    df_prepend = traindf[common_cols]
+            #else:
+
+
+            ### Make sure that the column orders are the same ####
+            #dfxs, _, _  = self.df_to_supervised(ts_df=df_prepend, drop_zero_var=False)
+            #dfxs = dfxs.tail(len(testdata_with_dummy))
+            #X_test = dfxs[self.transformed_preds]
+
             for i in np.arange(testdata_with_dummy.shape[0]):
 
                 # Append the last n_lags of the data to the row of the X_egogen that is being preducted
                 # Note that some of this will come from the last few observations of the training data
                 # and the rest will come from the last few observations of the X_egogen data.
                 # print(f"Prepend shape before adding test: {df_prepend.shape}")
+
                 df_prepend = df_prepend.append(testdata_with_dummy.iloc[i])
+                df_prepend.index = pd.to_datetime(df_prepend.index)
+
                 # print(f"Prepend shape after adding test: {df_prepend.shape}")
                 # print("Prepend Dataframe")
                 # print(df_prepend)
 
                 # Convert the appended dataframe to supervised learning problem
+
                 dfxs, _, _  = self.df_to_supervised(ts_df=df_prepend, drop_zero_var=False)
 
+
                 # Select only the predictors (transformed) from here
                 X_test = dfxs[self.transformed_preds]
                 # print("X_test")
@@ -438,6 +535,8 @@ class BuildML(BuildBase):
             return res_frame
 
 ###############################################################################################
+import dask
+import dask.dataframe as dd
 def create_time_series_features(dft, targets, ts_column: Optional[str]=None, drop_zero_var: bool = False):
     """
     This creates between 8 and 10 date time features for each date variable.
@@ -446,6 +545,8 @@ def create_time_series_features(dft, targets, ts_column: Optional[str]=None, dro
     create all these features using just the date time column that you send in.
     It returns the entire dataframe with added variables as output.
     """
+
+    reset_index = False
     time_preds = [x for x in list(dft) if x != targets]
     dtf = dft[time_preds]
     dtf_target = dft[[targets]]
@@ -455,10 +556,14 @@ def create_time_series_features(dft, targets, ts_column: Optional[str]=None, dro
         if ts_column is None:
             reset_index = True
             ts_column = dtf.index.name
-            dtf.reset_index(inplace=True)
+            dtf = dtf.reset_index()
 
         ### In some extreme cases, date time vars are not processed yet and hence we must fill missing values here!
-        if dtf[ts_column].isnull().sum() > 0:
+        if type(dft) == dask.dataframe.core.DataFrame:
+            null_nums = dtf[ts_column].isnull().sum().compute()
+        else:
+            null_nums = dtf[ts_column].isnull().sum()
+        if  null_nums > 0:
             # missing_flag = True
             new_missing_col = ts_column + '_Missing_Flag'
             dtf[new_missing_col] = 0
@@ -469,25 +574,32 @@ def create_time_series_features(dft, targets, ts_column: Optional[str]=None, dro
             dtf[ts_column] = dtf[ts_column].astype(int)
 
         ### if we have already found that it was a date time var, then leave it as it is. Thats good enough!
-
-        items = dtf[ts_column].apply(str).apply(len).values
-        #### In some extreme cases,
-        if all(items[0] == item for item in items):
-            if items[0] == 4:
+        if type(dft) == dask.dataframe.core.DataFrame:
+            if dtf[ts_column].apply(len).compute()[0] == 4:
                 ### If it is just a year variable alone, you should leave it as just a year!
-                dtf[ts_column] = pd.to_datetime(dtf[ts_column],format='%Y')
+                dtf[ts_column] = dd.to_datetime(dtf[ts_column],format='%Y')
             else:
                 ### if it is not a year alone, then convert it into a date time variable
-                dtf[ts_column] = pd.to_datetime(dtf[ts_column], infer_datetime_format=True)
+                dtf = create_ts_features_dask(df=dtf, tscol=ts_column, drop_zero_var=drop_zero_var, return_original=True)
         else:
-            dtf[ts_column] = pd.to_datetime(dtf[ts_column], infer_datetime_format=True)
+            items = dtf[ts_column].apply(str).apply(len).values
+            #### In some extreme cases,
+            if all(items[0] == item for item in items):
+                if items[0] == 4:
+                    ### If it is just a year variable alone, you should leave it as just a year!
+                    dtf[ts_column] = pd.to_datetime(dtf[ts_column],format='%Y')
+                else:
+                    ### if it is not a year alone, then convert it into a date time variable
+                    dtf[ts_column] = pd.to_datetime(dtf[ts_column], infer_datetime_format=True)
+            else:
+                dtf[ts_column] = pd.to_datetime(dtf[ts_column], infer_datetime_format=True)
 
-        dtf = create_ts_features(df=dtf, tscol=ts_column, drop_zero_var=drop_zero_var, return_original=True)
+                dtf = create_ts_features(df=dtf, tscol=ts_column, drop_zero_var=drop_zero_var, return_original=True)
 
         # If you had reset the index earlier, set it back before returning
         # to  make it consistent with the dataframe that was sent as input
         if reset_index:
-            dtf.set_index(ts_column, inplace=True)
+            dtf = dtf.set_index(ts_column)
 
     except Exception as e:
         print(e)
@@ -498,6 +610,61 @@ def create_time_series_features(dft, targets, ts_column: Optional[str]=None, dro
         print('Error in creating time-series features...Continuing')
     return dtf
 ##############################################################################################
+def create_ts_features_dask(
+    df,
+    tscol,
+    drop_zero_var: bool = True,
+    return_original: bool = True) -> pd.DataFrame:
+    """
+    This takes in input a DASK or pandas dataframe and a date time index - if not it will fail
+
+    :param drop_zero_var If True, it will drop any features that have zero variance
+    :type drop_zero_var bool
+
+    :param return_original If True, it will return the original dataframe concatenated with the derived features
+    else, it will just return the derived features
+    :type return_original bool
+
+    :rtype pd.DataFrame
+    """
+    df_org = copy.deepcopy(df)
+    dt_adds = []
+    try:
+        df[tscol+'_hour'] = df.index.hour.values
+        dt_adds.append(tscol+'_hour')
+    except:
+        print('    Error in creating hour time feature. Continuing...')
+    try:
+        df[tscol+'_minute'] = df.index.minute.values
+        dt_adds.append(tscol+'_minute')
+    except:
+        print('    Error in creating minute time feature. Continuing...')
+    try:
+        df[tscol+'_dayofweek'] = df.index.dayofweek.values
+        dt_adds.append(tscol+'_dayofweek')
+        df[tscol+'_quarter'] = df.index.quarter.values
+        dt_adds.append(tscol+'_quarter')
+        df[tscol+'_month'] = df.index.month.values
+        dt_adds.append(tscol+'_month')
+        df[tscol+'_year'] = df.index.year.values
+        dt_adds.append(tscol+'_year')
+        df[tscol+'_dayofyear'] = df.index.dayofyear.values
+        dt_adds.append(tscol+'_dayofyear')
+        df[tscol+'_dayofmonth'] = df.index.day.values
+        dt_adds.append(tscol+'_dayofmonth')
+        df[tscol+'_weekofyear'] = df.index.weekofyear.values
+        dt_adds.append(tscol+'_weekofyear')
+        weekends = (df[tscol+'_dayofweek'] == 5) | (df[tscol+'_dayofweek'] == 6)
+        df[tscol+'_weekend'] = 0
+        df[tscol+'_weekend'] = df[tscol+'_weekend'].mask(weekends, 1)
+        dt_adds.append(tscol+'_weekend')
+    except:
+        print('    Error in creating date time derived features. Continuing...')
+
+    #df = df[dt_adds].fillna(0).astype(int)
+
+    return df
+################################################################################
 def create_ts_features(
     df,
     tscol,
